@@ -107,7 +107,7 @@ def split_functions(fpgm_tokens):
     tokens = iter(fpgm_tokens)
     for t in tokens:
         mnemonic = t.mnemonic
-        if mnemonic.startswith("FDEF"):
+        if mnemonic.startswith("FDEF") or mnemonic.startswith("IDEF"):
             body = [t]
             for t in tokens:
                 body.append(t)
@@ -127,6 +127,7 @@ def merge_functions(functions, include=None):
     for line in "\n".join(functions).splitlines():
         asm.extend(line.strip().split())
     funcs = {}
+    idefs = {}
     stack = []
     tokens = iter(asm)
     for token in tokens:
@@ -137,14 +138,18 @@ def merge_functions(functions, include=None):
                     stack.append(num)
                 except ValueError:
                     break
-        if token.startswith("FDEF"):
+        if token.startswith(("FDEF", "IDEF")):
+            is_fdef = token.startswith("FDEF")
             num = stack.pop()
             body = [token]
             for token in tokens:
                 body.append(token)
                 if token.startswith("ENDF"):
                     break
-            funcs[num] = body
+            if is_fdef:
+                funcs[num] = body
+            else:
+                idefs[num] = body
             continue
         assert 0, "Unexpected token in fpgm: %s" % token
     if include is not None:
@@ -153,6 +158,10 @@ def merge_functions(functions, include=None):
     result = ["PUSH[]"] + [str(n) for n in sorted(funcs, reverse=True)]
     for num in sorted(funcs):
         result.extend(funcs[num])
+    result.append("PUSH[]")
+    result.extend([str(n) for n in sorted(idefs, reverse=True)])
+    for num in sorted(idefs):
+        result.extend(idefs[num])
     return result
 
 
@@ -311,7 +320,24 @@ def transform(tokens, components=None):
                 for i in reversed(t.stack_items):
                     stream[push_indexes[-1]].appendleft(i)
             else:
-                assert not t.stack_items
+                assert not t.stack_items or mnemonic.startswith(("PUSH", "NPUSH"))
+
+                # While it is true that most instructions cannot take stack
+                # parameters in #PUSHOFF mode, you can have PUSHx[] and NPUSHx[]
+                # instructions that do have parameters that need to get onto the
+                # stack. (ok, technically, they are not parameters to put on the
+                # stack but data items in the instruction stream).
+                # If the current instruction is a kind of PUSH, emit the
+                # instruction with its stack parameters (excluding the count for
+                # NPUSHx instructions as fontTools computes that).
+                if t.stack_items:
+                    if mnemonic.startswith("NPUSH"):
+                        params = t.stack_items[1:]
+                    elif mnemonic.startswith("PUSH"):
+                        params = t.stack_items[:]
+                    stream.append([f"{mnemonic}[]"] + params)
+                    pos += 1
+                    continue
 
         stream.append(["{}[{}]".format(mnemonic, t.flags)])
         pos += 1
